/* ----------------------------------------------

   EPWING ҥե(HONMON) 饤֥

   Copyright (c) 2001-2002 Keichiro Katayama
                    keichiro@ike.tottori-u.ac.jp

   ---------------------------------------------- */  

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "epwing.h"
#include "honmon.h"

EPW_HON_HEADER   header;
EPW_HON_ITEMINFO *iteminfo;
EPW_HON_DISPINFO dispinfo;

EPW_HON_BODYINFO bodyinfo;

FILE *fp_honmon;
char honmon_filename[256];

EPW_HON_RECORD record;

static int honmonReadBlock( EPW_HON_RECORD *HonmonRecord );
static int honmonParseInfoBlock( EPW_HON_RECORD *HonmonRecord );
static int honmonRecordCopy( void *dest, EPW_HON_RECORD *HonmonRecord, const int length );

int honmonOpen( const char *filename )
{

	if ( fp_honmon != NULL ) {
		#ifdef DEBUG
			fprintf( stderr, "honmon.c: honmonOpen() HONMON has already been opened. Use honmonClose first.\n" );
		#endif
		return EPWING_ERROR;
	}

	if ( ( fp_honmon = fopen( filename, "rb" ) ) == NULL ) {
		fprintf( stderr, "Cannot open \"%s\" \n", filename );
		return EPWING_ERROR;
	}
	strcpy( honmon_filename, filename );

	record.block = 1;
	honmonReadBlock( &record );

	return honmonParseInfoBlock( &record );
}

int honmonClose( void )
{
	if ( fp_honmon == NULL ) {
		return EPWING_SUCCESS;
	}

	fclose( fp_honmon );
	free( iteminfo );
		
	return EPWING_SUCCESS;
}


/* HonmonRecord.block ǻꤵ줿֥å HonmonRecord.data[] ɤ߹ */
static int honmonReadBlock( EPW_HON_RECORD *HonmonRecord )
{
	if ( fseek( fp_honmon, ( HonmonRecord->block - 1 ) * EPWING_BLOCK_SIZE, SEEK_SET ) != 0 ) {
		#ifdef DEBUG
			fprintf( stderr, "honmon.c: honmonReadBlock() fseek was failed.\n" );
		#endif
		return EPWING_ERROR;
	}

	fread( HonmonRecord->data, EPWING_BLOCK_SIZE, 1, fp_honmon );
	HonmonRecord->pos = HonmonRecord->data;

	return EPWING_SUCCESS;
}

int honmonMoveBlock( const int blockno )
{
	record.block = blockno;
	return honmonReadBlock( &record );
}

int honmonGetCurrentBlock( void )
{
	return record.block;
}

/* HonmonRecord.pos  length byte dest˥ԡơݥ(pos) length ʬʤ롣 */
static int honmonRecordCopy( void *dest, EPW_HON_RECORD *HonmonRecord, const int length )
{
	if ( ( EPWING_BLOCK_SIZE - ( (unsigned int)HonmonRecord->pos - (unsigned int)HonmonRecord->data ) ) < (unsigned int)length ) {
		#ifdef DEBUG
			fprintf( stderr, "honmon.c: honmonRecordCopy() Block alignment over....\n" );
		#endif
		return EPWING_ERROR;
	}
	memcpy( dest, HonmonRecord->pos, length );

	HonmonRecord->pos += length;
	if ( ( (unsigned int)HonmonRecord->pos - (unsigned int)HonmonRecord->pos  ) >= EPWING_BLOCK_SIZE )  {
		HonmonRecord->block++;

		#ifdef DEBUG
			fprintf( stderr, "honmonGetWord: Move Next Block(%d)\n", record.block );
			if( honmonReadBlock( &record ) != EPWING_SUCCESS ) {
				fprintf( stderr, "honmonGetWord: honmonReadBlock() failed.\n" );
			}
		#else
			honmonReadBlock( &record );
		#endif
	}


	return EPWING_SUCCESS;
}

int honmonGetWord( void )
{
	int w;

	w = epwGetWord( record.pos );

	record.pos += sizeof( WORD );
	if ( ( (unsigned int)record.pos - (unsigned int)record.data ) >= EPWING_BLOCK_SIZE )  {
		record.block++;

		#ifdef DEBUG
			fprintf( stderr, "honmonGetWord: Move Next Block(%d)\n", record.block );
			if( honmonReadBlock( &record ) != EPWING_SUCCESS ) {
				fprintf( stderr, "honmonGetWord: honmonReadBlock() failed.\n" );
			}
		#else
			honmonReadBlock( &record );
		#endif
	}

	return w;
}

/* length * 2(=sizeof(WORD)) byte ʬݥ󥿤ʤ롣 */
void honmonMovePos( const int length )
{
	int i;

	for ( i = 0; i < length; i++ ) {
		record.pos += sizeof( WORD );
		if ( ( (unsigned int)record.pos - (unsigned int)record.data ) >= EPWING_BLOCK_SIZE )  {
			record.block++;

			#ifdef DEBUG
				fprintf( stderr, "honmonMovePos: Move Next Block(%d)\n", record.block );
				if( honmonReadBlock( &record ) != EPWING_SUCCESS ) {
					fprintf( stderr, "honmonMovePos: honmonReadBlock() failed.\n" );
				}
			#else
				honmonReadBlock( &record );
			#endif
		}
	}
}

/* HonmonRecord ¸Ƥ֥åҴȤƲϤ */
static int honmonParseInfoBlock( EPW_HON_RECORD *HonmonRecord )
{
	int itemcount;
	int i;

	/* Ҵ֥å */
	honmonRecordCopy( &header, HonmonRecord, sizeof( EPW_HON_HEADER ) );

	itemcount = epwGetWord( header.itemcount );

	/* ҹǾɤ߽Ф */
	if ( iteminfo != NULL ) {
		free( iteminfo );
	}
	iteminfo = (EPW_HON_ITEMINFO *)malloc( itemcount * sizeof( EPW_HON_ITEMINFO ) );
	if ( iteminfo == NULL ) {
		#ifdef DEBUG
			fprintf( stderr, "honmon.c: honmonParseInfoBlock() malloc failed.\n" );
		#endif
		return EPWING_ERROR;
	}
	honmonRecordCopy( iteminfo, HonmonRecord, sizeof( EPW_HON_ITEMINFO) * itemcount );

	/* Ҵ֥å */
	honmonRecordCopy( &dispinfo, HonmonRecord, sizeof( EPW_HON_DISPINFO ) );


	/* ʸ(id=0x00)Υ֥å˴ؤθ */
	bodyinfo.startblock = 0;
	for ( i = 0; i < itemcount; i++ ) {
		if ( iteminfo[i].id == 0x00 ) {
			bodyinfo.startblock = epwGetLong( iteminfo[i].startblock );
			bodyinfo.size = epwGetLong( iteminfo[i].size );
			bodyinfo.lastblock = bodyinfo.startblock + bodyinfo.size - 1;

			#ifdef DEBUG
				fprintf( stderr, "honmonParseInfoBlock: Body Found!\n" );
			#endif

			break;
		}
	}

	#ifdef DEBUG
		if ( bodyinfo.startblock == 0 ) {
			fprintf( stderr, "honmon.c: honmonParseInfoBlock() Couldn't find Body.\n" );
		}
	#endif

	return EPWING_SUCCESS;
}

EPW_HON_HEADER *honmonGetHeader( void )
{
	return &header;
}

EPW_HON_ITEMINFO *honmonGetItemInfo( void )
{
	return iteminfo;
}

EPW_HON_DISPINFO *honmonGetDispInfo( void )
{
	return &dispinfo;
}

EPW_HON_BODYINFO *honmonGetBodyInfo( void )
{
	return &bodyinfo;
}
