/* ----------------------------------------------

   ɤ˴ؤ饤֥

   Copyright (c) 2001-2002 Keichiro Katayama
                    keichiro@ike.tottori-u.ac.jp

   ---------------------------------------------- */

#include "kanji.h"

/* JIS code to EUC code */
void jis2euc( unsigned char *dest, const unsigned char *s, const int length )
{
	unsigned char *d;
	int i = 0;

	d = dest;
	
	while ( *s && i < length ) {
		*d = *s | 0x80;

		s++;
		d++;
		i++;
	}
	*d = 0;
}

/* JIS code to sjis code */
void jis2sjis( unsigned char *dest, const unsigned char *s, const int length )
{
	unsigned char *d;
	unsigned char low, high;
	int i = 0;

	d = dest;

	while ( *s  && i < length ) {
		high = *s;
		s++;
		low = *s;

		if ( high & 0x01 ) {
			low += 0x1f;
		} else {
			low += 0x7d;
		}

		if ( low >= 0x7f ) {
			low++;
		}

		high = ( ( high - 0x21 ) >> 1 ) + 0x81;

		if ( high > 0x9f ) {
			high += 0x40;
		}

		*d = high;
		d++;
		*d = low;
		
		s++;
		d++;
		i+=2;
	}
	*d = 0;
}

unsigned char jis2ascii( const unsigned int jiscode )
{

	/* Kigo Mapping Table          */
	/* 0x2121 - 0x215f             */
	static const unsigned char JisToAsciiTable[] = { 
		0x00, /* Dummy */
		0x20, /*  */
		0x00, /*  */
		0x00, /*  */
		0x2C, /*  */
		0x2E, /*  */
		0x00, /*  */
		0x3A, /*  */
		0x3B, /*  */
		0x3F, /*  */
		0x21, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x60, /*  */
		0x00, /*  */
		0x5E, /*  */
		0x00, /*  */
		0x5F, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x2F, /*  */
		0x00, /*  */
		0x7E, /*  */
		0x00, /*  */
		0x7C, /*  */
		0x00, /*  */
		0x00, /*  */
		0x27, /*  */
		0x27, /*  */
		0x22, /*  */
		0x22, /*  */
		0x28, /*  */
		0x29, /*  */
		0x00, /*  */
		0x00, /*  */
		0x5B, /*  */
		0x5D, /*  */
		0x7B, /*  */
		0x7D, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x2B, /*  */
		0x2D, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x3D, /*  */
		0x00, /*  */
		0x3C, /*  */
		0x3E, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x5C, /*  */
		0x24, /*  */
		0x00, /*  */
		0x00, /*  */
		0x25, /*  */
		0x23, /*  */
		0x26, /*  */
		0x2A, /*  */
		0x40, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
		0x00, /*  */
	};

	/* Kigo */
	if ( 0x2121 <= jiscode && jiscode <= 0x217f ) {
		return JisToAsciiTable[ ( jiscode & 0xff ) - 0x20 ];
    }

	/* 0-9                         */
	/* 0x2330-0x2339 -> 0x30-0x39  */
	/* A - Z                       */
	/* 0x2341-0x235a -> 0x41-0x5a  */
	/* a - z                       */
	/* 0x2361-0x237a -> 0x61-0x7a  */
	if ( ( 0x2330 <= jiscode && jiscode <= 0x2339 ) ||
		 ( 0x2341 <= jiscode && jiscode <= 0x235a ) ||
		 ( 0x2361 <= jiscode && jiscode <= 0x237a ) ) {
		return (unsigned char)( jiscode & 0xff );
	}

	return 0x00;
}
